#include "ShaderConstants.fxh"
#include "Util.fxh"
#include "lib/option.fxh"

struct PS_Input {
	float4 position : SV_Position;
        float3 poss : POSS;
        float3 wpos : WPOS;

#ifdef ENABLE_LIGHT
	float4 light : LIGHT;
#endif
#ifdef ENABLE_FOG
	float4 fogColor : FOG_COLOR;
#endif

#ifndef DISABLE_TINTING
	float4 color : COLOR;
#endif

	float4 texCoords : TEXCOORD_0_FB_MSAA;
};

struct PS_Output
{
	float4 color : SV_Target;
};

float waterWave(float3 p, float tm){
      float a = noise(float2(tm*.4+(p.x+p.z)/3.,tm*.4+(p.x+p.z)/3.));
      float b = noise(float2(tm*.6+(p.z+p.x)/3.5,tm*.6+(p.z+p.x)/3.5));
      float interp = sin(cos(p.x+p.z)+p.z+tm);
      float wave = lerp(a*.55,b*.8,interp+.3);
   return wave;
}

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{       detection at;     //}
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.texCoords.xy);
	float4 base = TEXTURE_0.Sample(TextureSampler0, PSInput.texCoords.zw);

#ifndef DISABLE_TINTING
	base.a = lerp(diffuse.r * diffuse.a, diffuse.a, PSInput.color.a);
	base.rgb *= PSInput.color.rgb;
#endif

#ifdef ENABLE_LIGHT
	base.rgb *= PSInput.light.rgb;
#endif

// Supplement variables
  float rendis = RENDER_DISTANCE;
  float fogdis = autoDistance(PSInput.wpos/rendis*3.);
  float fogRain = autoDistance(PSInput.wpos/rendis*4.3);
  float2 fcc = FOG_CONTROL;
  float4 fc = FOG_COLOR;
  float lookSun =  saturate((1.-fc.b)*1.5)*saturate((fc.r -.15)*1.25);
  bool rain_s = rain_f(fc);

// Time set
    at.day = saturate((PSInput.light.r)*2.);
    at.rain = 1.-smoothstep(.3, .66, fcc.x);
    at.cave = smoothstep(.9,.8, fc.b);
    at.night = pow(max(min(1.-fc.r*1.5,1.),.0),1.2);
    at.dusk = pow(max(min(1.-fc.b*1.2,1.), 0.),.5);
    at.cave *= (1.-at.night);
    at.night *= (1.-at.cave);
    at.rain *= float(rain_s);
    
    // separate detection color
    float rain2 = 1.-smoothstep(.3, .66, fcc.x);

// normal | direction
    float3 dir = normalize(cross(ddx(PSInput.poss),ddy(PSInput.poss)));

// entity color
float3 clr = lerp(float3(1.,1.,1.), float3(1.4,1.45,1.6), at.day);
         clr = lerp(clr, lerp(float3(.85,.85,.85), float3(1.,1.,1.), max(at.night, at.dusk)), at.cave);
         clr = lerp(clr, lerp(float3(1.3,1.,-.5), float3(1.,1.,1.), rain2), at.dusk);
         clr = lerp(clr, float3(.8,.8,.8), at.rain);
         clr = lerp(clr, lerp(float3(.9,.9,1.), float3(1.,1.,1.), at.dusk), at.night);

// underwater color
    if(isUnderwater(fcc, fc)){
    if(REALISTIC_WATER == ON && VANILLA == OFF){

      float getWave = abs(waterWave(PSInput.poss.zyx*.05, TIME*.8));
      float3 caustic = lerp(float3(.0,.0,.0), float3(.8,1.,1.2)*6.,getWave);
      base.rgb *= float3(.4,.5,.6);
      base.rgb += lerp(float3(.0,.0,.0), caustic, fc.b-.4)*base.rgb;
      base.rgb += lerp(float3(.0,.0,.0), float3(.05,.1,.2), fogdis);
   }
}

if(!(isUnderwater(fcc,fc) || isTheEnd(fc) || isNether(fcc))){
  base.rgb = tonemap(base.rgb,1.);
  base.rgb *= clr;
  base.rgb = saturation(base.rgb,1.25);
}

// nether color
    if(isNether(fcc)){
       base.rgb *= float3(1.,.65,.0);
       base.rgb -= lerp(float3(.0,.0,.0), float3(.15,.1,.0),fogdis);
}

// the end color
    if(isTheEnd(fc)){
      
      if(GALAXY == OFF){
    base.rgb *= float3(1.5,1.,2.7);
    base.rgb += lerp(float3(.0,.0,.0), float3(.17,.11,.235),fogdis);

                    }else{

      base.rgb *= float3(.5,.5,1.3);
      base.rgb -= lerp(float3(.0,.0,.0), float3(.25,.25,.25), fogdis);
   }
}

// dirlight
   if(!isTheEnd(fc)){
   base.rgb *= lerp(lerp(1., lerp(1., lerp(.4,.65, max(0., max(at.rain, at.dusk))), abs(dir.x)), at.day), 1., autoPos(PSInput.wpos/rendis*2.));
   base.rgb *= lerp(lerp(float3(1.,1.,1.), lerp(float3(1.,1.,1.), lerp(float3(2.,1.54,-1.)*1.5, float3(1.,1.,1.), max(0.,max(at.night, max(at.cave, rain2)))), abs(dir.z)), at.dusk), float3(1.,1.,1.), autoPos(PSInput.wpos/rendis*.4));}

// fog entity
if(ATMOSPHERE == ON){
if(!(isNether(fcc) || isTheEnd(fc) || isUnderwater(fcc, fc))){
   //base.rgb -= lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(2.5,2.5,2.5), float3(.0,.0,.0), max(at.night, max(at.rain, at.dusk))), at.cave), fogdis);
   base.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), fc.rgb*float3(1.1,1.1,1.1), at.dusk*lookSun), fogdis);
   base.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(.8,.8,.8), float3(.0,.0,.0), max(at.night, max(at.rain, at.cave))), at.dusk), fogdis);
   base.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(.08,.08,.08), float3(.0,.0,.0), max(at.cave, max(at.rain, lookSun))), at.night), fogdis);
   base.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(.45,.45,.45), float3(.15,.15,.15), at.night), at.rain), fogRain);
  }
}

#ifdef ENABLE_FOG
	//apply fog
	base.rgb = lerp(base.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif

	//WARNING do not refactor this 
	PSOutput.color = base;
#ifdef UI_ENTITY
	PSOutput.color.a *= HUD_OPACITY;
#endif

#ifdef VR_FEATURE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif
}