#include "ShaderConstants.fxh"
#include "lib/option.fxh"

struct VS_Input
{
    float3 position : POSITION;
    float4 color : COLOR;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif

};

struct PS_Input
{
    float4 position : SV_Position;
    float4 color : COLOR;
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};

static const float fogNear = 0.9;
static const float3 inverseLightDirection = float3( 0.62, 0.78, 0.0 );
static const float ambient = 0.7;

ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput){

   float4 cc = CURRENT_COLOR;
   float4 fc = FOG_COLOR;
   float2 fcc = FOG_CONTROL;
   float3 poss = VSInput.position.xyz;

detection at;
at.day = pow(max(min(1.-fc.b*1.2,1.), .0), .5);
at.rain = 1.-smoothstep(.3,.66,fcc.x);

float fade1 = 1.*(1.1-at.day);
float fade2 = 1.*(1.2-at.rain);

#ifdef INSTANCEDSTEREO
	int i = VSInput.instanceID;
        if(VANILLA == OFF){
	PSInput.position = mul( WORLDVIEWPROJ_STEREO[i], (float4( VSInput.position, 1 )*float4(1.,3.,1.,1.)));
        }else{
        PSInput.position = mul( WORLDVIEWPROJ_STEREO[i], (float4( VSInput.position, 1 )*float4(1.,1.,1.,1.)));}
	float3 worldPos = mul(WORLD_STEREO, float4(VSInput.position, 1));
#else
        if(VANILLA == OFF){
	PSInput.position = mul(WORLDVIEWPROJ, (float4(VSInput.position, 1)*float4(1.,3.,1.,1.)));
        }else{
	PSInput.position = mul(WORLDVIEWPROJ, (float4(VSInput.position, 1)*float4(1.,1.,1.,1.)));}
	float3 worldPos = mul(WORLD, float4(VSInput.position, 1));
#endif


#ifdef GEOMETRY_INSTANCEDSTEREO
	PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	PSInput.renTarget_id = VSInput.instanceID;
#endif

           float4 cloudc;
           if(NO_CLOUD == OFF){
           cloudc = float4(.7,.8,.9,.4);}
           PSInput.color = cloudc;
           PSInput.color.rgb += float3(.4,.4,.4)*poss.y;
           PSInput.color.a *= fade1;
           PSInput.color.a *= fade2;

    float depth = length(worldPos) / RENDER_DISTANCE;
    float fog = max( depth - fogNear, 0.0 );

    PSInput.color.a *= 1.0 - fog;
}