#include "ShaderConstants.fxh"
#include "util.fxh"
#include "lib/option.fxh"

struct PS_Input {
	float4 position : SV_Position;
	float4 light : LIGHT;
	float4 fogColor : FOG_COLOR;
        float3 poss : POSS;
        float3 wpos : WPOS;

#ifdef GLINT
	// there is some alignment issue on the Windows Phone 1320 that causes the position
	// to get corrupted if this is two floats and last in the struct memory wise
	float4 layerUV : GLINT_UVS;
#endif

#ifdef COLOR_BASED
	float4 color : COLOR;
#endif

#ifdef USE_OVERLAY
	float4 overlayColor : OVERLAY_COLOR;
#endif

#ifdef TINTED_ALPHA_TEST
	float4 alphaTestMultiplier : ALPHA_MULTIPLIER;
#endif

	float2 uv : TEXCOORD_0_FB_MSAA;

};

struct PS_Output
{
	float4 color : SV_Target;
};

#ifdef USE_EMISSIVE
#ifdef USE_ONLY_EMISSIVE
#define NEEDS_DISCARD(C) (C.a == 0.0f ||C.a == 1.0f )
#else
#define NEEDS_DISCARD(C)	(C.a + C.r + C.g + C.b == 0.0)
#endif
#else
#ifndef USE_COLOR_MASK
#define NEEDS_DISCARD(C)	(C.a < 0.5)
#else
#define NEEDS_DISCARD(C)	(C.a == 0.0)
#endif
#endif

float4 glintBlend(float4 dest, float4 source) {
	// glBlendFuncSeparate(GL_SRC_COLOR, GL_ONE, GL_ONE, GL_ZERO)
	return float4(source.rgb * source.rgb, source.a) + float4(dest.rgb, 0.0);
}

float waterWave(float3 p, float tm){
      float a = noise(float2(tm*.4+(p.x+p.z)/3.,tm*.4+(p.x+p.z)/3.));
      float b = noise(float2(tm*.6+(p.z+p.x)/3.5,tm*.6+(p.z+p.x)/3.5));
      float interp = sin(cos(p.x+p.z)+p.z+tm);
      float wave = lerp(a*.55,b*.8,interp+.3);
   return wave;
}

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{ detection at; //}
	float4 color = float4( 1.0f, 1.0f, 1.0f, 1.0f );

#if( !defined(NO_TEXTURE) || !defined(COLOR_BASED) || defined(USE_COLOR_BLEND) )

#if !defined(TEXEL_AA) || !defined(TEXEL_AA_FEATURE) || (VERSION < 0xa000 /*D3D_FEATURE_LEVEL_10_0*/)
	color = TEXTURE_0.Sample( TextureSampler0, PSInput.uv );
#else
	color = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv);
#endif

#ifdef MASKED_MULTITEXTURE
	float4 tex1 = TEXTURE_1.Sample(TextureSampler1, PSInput.uv);

	// If tex1 has a non-black color and no alpha, use color; otherwise use tex1 
	float maskedTexture = ceil( dot( tex1.rgb, float3(1.0f, 1.0f, 1.0f) ) * ( 1.0f - tex1.a ) );
	color = lerp(tex1, color, saturate(maskedTexture));
#endif // MASKED_MULTITEXTURE

#if defined(ALPHA_TEST) && !defined(USE_MULTITEXTURE) && !defined(MULTIPLICATIVE_TINT)
	if( NEEDS_DISCARD( color ) )
	{
		discard;
	}
#endif

#ifdef TINTED_ALPHA_TEST
	float4 testColor = color;
	testColor.a = testColor.a * PSInput.alphaTestMultiplier.r;
	if( NEEDS_DISCARD( testColor ) )
	{
		discard;
	}
#endif

#endif

#ifdef COLOR_BASED
	color *= PSInput.color;
#endif

#ifdef MULTI_COLOR_TINT
	// Texture is a mask for tinting with two colors
	float2 colorMask = color.rg;

	// Apply the base color tint
	color.rgb = colorMask.rrr * CHANGE_COLOR.rgb;

	// Apply the secondary color mask and tint so long as its grayscale value is not 0
	color.rgb = lerp(color.rgb, colorMask.ggg * MULTIPLICATIVE_TINT_CHANGE_COLOR.rgb, ceil(colorMask.g));
#else

#ifdef USE_COLOR_MASK
	color.rgb = lerp( color, color * CHANGE_COLOR, color.a ).rgb;
	color.a *= CHANGE_COLOR.a;
#endif

#ifdef ITEM_IN_HAND
	color.rgb = lerp(color, color * CHANGE_COLOR, color.a).rgb;
#endif

#endif

#ifdef USE_MULTITEXTURE
	float4 tex1 = TEXTURE_1.Sample(TextureSampler1, PSInput.uv);
	float4 tex2 = TEXTURE_2.Sample(TextureSampler2, PSInput.uv);
	color.rgb = lerp(color.rgb, tex1, tex1.a);
#ifdef ALPHA_TEST
	if (color.a < 0.5f && tex1.a == 0.0f) {
		discard;
	}
#endif

#ifdef COLOR_SECOND_TEXTURE
	if (tex2.a > 0.0f) {
		color.rgb = lerp(tex2.rgb, tex2 * CHANGE_COLOR, tex2.a);
	}
#else
	color.rgb = lerp(color.rgb, tex2, tex2.a);
#endif
#endif

#ifdef MULTIPLICATIVE_TINT
	float4 tintTex = TEXTURE_1.Sample(TextureSampler1, PSInput.uv);

#ifdef MULTIPLICATIVE_TINT_COLOR 
	tintTex.rgb = tintTex.rgb * MULTIPLICATIVE_TINT_CHANGE_COLOR.rgb;
#endif

#ifdef ALPHA_TEST
	color.rgb = lerp(color.rgb, tintTex.rgb, tintTex.a);
	if (color.a + tintTex.a <= 0.0f) {
		discard;
	}
#endif
#endif

#ifdef USE_OVERLAY
	//use either the diffuse or the OVERLAY_COLOR
	color.rgb = lerp( color, PSInput.overlayColor, PSInput.overlayColor.a ).rgb;
#endif

#ifdef USE_EMISSIVE
	//make glowy stuff
	color *= lerp( float( 1.0 ).xxxx, PSInput.light, color.a );
#else
	color *= PSInput.light;
#endif

// supplement variables
  float rendis = RENDER_DISTANCE;
  float fogdis = autoDistance(PSInput.wpos/rendis*3.);
  float fogRain = autoDistance(PSInput.wpos/rendis*4.3);
  float2 fcc = FOG_CONTROL;
  float4 fc = FOG_COLOR;
  float lookSun =  saturate((1.-fc.b)*1.5)*saturate((fc.r -.15)*1.25);
  bool rain_s = rain_f(fc);

// Time set
    at.day = saturate((PSInput.light.r)*2.);
    at.rain = 1.-smoothstep(.3, .66, fcc.x);
    at.cave = smoothstep(.9,.8,fc.b);
    at.night = pow(max(min(1.-fc.r*1.5,1.),.0),1.2);
    at.dusk = pow(max(min(1.-fc.b*1.2,1.), 0.),.5);
    at.cave *= (1.-at.night);
    at.night *= (1.-at.cave);
    at.rain *= float(rain_s);
    float rain2 = 1.-smoothstep(.3, .66, fcc.x);

// normal | direction
    float3 dir = normalize(cross(ddx(PSInput.poss),ddy(PSInput.poss)));

// entity color
float3 clr = lerp(float3(1.,1.,1.), float3(1.4,1.45,1.6), at.day);
         clr = lerp(clr, lerp(float3(.85,.85,.85), float3(1.,1.,1.), max(at.night, at.dusk)), at.cave);
         clr = lerp(clr, lerp(float3(1.3,1.,-.5), float3(1.,1.,1.), rain2), at.dusk);
         clr = lerp(clr, float3(.8,.8,.8), at.rain);
         clr = lerp(clr, lerp(float3(.9,.9,1.), float3(1.,1.,1.), at.dusk), at.night);

// underwater color
    if(isUnderwater(fcc, fc)){
    if(REALISTIC_WATER == ON && VANILLA == OFF){

      float getWave = abs(waterWave(PSInput.poss.zyx*.15, TIME*.8));
      float3 caustic = lerp(float3(.0,.0,.0), float3(.8,1.,1.2)*8.,getWave);
      color.rgb *= float3(.4,.5,.6);
      color.rgb += lerp(float3(.0,.0,.0), caustic, fc.b-.4)*color.rgb;
      color.rgb += lerp(float3(.0,.0,.0), float3(.05,.1,.2), fogdis);
   }
}

if(!(isUnderwater(fcc,fc) || isTheEnd(fc) || isNether(fcc))){
  color.rgb = tonemap(color.rgb,1.);
  color.rgb *= clr;
  color.rgb = saturation(color.rgb,1.25);
}

// nether color
    if(isNether(fcc)){
       color.rgb *= float3(1.,.65,.0);
       color.rgb -= lerp(float3(.0,.0,.0), float3(.15,.1,.0),fogdis);
}

// the end color
    if(isTheEnd(fc)){
      
      if(GALAXY == OFF){
    color.rgb *= float3(1.5,1.,2.7);
    color.rgb += lerp(float3(.0,.0,.0), float3(.17,.11,.235),fogdis);

                    }else{

      color.rgb *= float3(.5,.5,1.3);
      color.rgb -= lerp(float3(.0,.0,.0), float3(.25,.25,.25), fogdis);
   }
}

// dirlight
   if(!isTheEnd(fc)){
   color.rgb *= lerp(lerp(1., lerp(1., lerp(.4,.65, max(0., max(at.rain, at.dusk))), abs(dir.x)), at.day), 1., autoPos(PSInput.wpos/rendis*2.));
   color.rgb *= lerp(lerp(float3(1.,1.,1.), lerp(float3(1.,1.,1.), lerp(float3(2.,1.54,-1.)*1.5, float3(1.,1.,1.), max(0.,max(at.night, max(at.cave, rain2)))), abs(dir.z)), at.dusk), float3(1.,1.,1.), autoPos(PSInput.wpos/rendis*.4));}

// fog entity
if(ATMOSPHERE == ON){
if(!(isNether(fcc) || isTheEnd(fc) || isUnderwater(fcc, fc))){
   //color.rgb -= lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(2.5,2.5,2.5), float3(.0,.0,.0), max(at.night, max(at.rain, at.dusk))), at.cave), fogdis);
   color.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(fc.rgb*float3(1.1,1.1,1.1), float3(.0,.0,.0), at.dusk), at.dusk*lookSun), fogdis);
   color.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(.8,.8,.8), float3(.0,.0,.0), max(at.night, max(at.rain, at.cave))), at.dusk), fogdis);
   color.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(.08,.08,.08), float3(.0,.0,.0), max(at.cave, max(at.rain, lookSun))), at.night), fogdis);
   color.rgb += lerp(float3(.0,.0,.0), lerp(float3(.0,.0,.0), lerp(float3(.45,.45,.45), float3(.15,.15,.15), at.night), at.rain), fogRain);
  }
}

	//apply fog
	color.rgb = lerp( color.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );

#ifdef GLINT
	// Applies color mask to glint texture instead and blends with original color
	float4 layer1 = TEXTURE_1.Sample(TextureSampler1, frac(PSInput.layerUV.xy)).rgbr * GLINT_COLOR;
	float4 layer2 = TEXTURE_1.Sample(TextureSampler1, frac(PSInput.layerUV.zw)).rgbr * GLINT_COLOR;
	float4 glint = (layer1 + layer2) * TILE_LIGHT_COLOR;
	color = glintBlend(color, glint);
#endif

	//WARNING do not refactor this 
	PSOutput.color = color;
#ifdef UI_ENTITY
	PSOutput.color.a *= HUD_OPACITY;
#endif

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif
}