#include "ShaderConstants.fxh"
#include "util.fxh"
#include "lib/option.fxh"

struct PS_Input
{
	float4 position : SV_Position;
        float3 poss : POSS;
        float3 wpos : WPOS;
        float3 screen : SCREEN;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

// water color
float3 clrWater(float3 clrWtr, float shx, detection at){
  clrWtr = lerp(float3(.05,.45,.6), float3(.0,.3,.5), at.dusk); 
  clrWtr = lerp(clrWtr, float3(.05,.05,.1), at.cave);
  clrWtr = lerp(clrWtr, float3(.1, .1, .13), at.rain);
  clrWtr = lerp(clrWtr, lerp(float3(.0, .05, .15), float3(.05,.05,.05), at.rain), at.night);
  clrWtr = lerp(clrWtr, float3(1.,.5,.0), mid(shx-.45));
 return clrWtr; 
}

// torch light
float3 light(float3 result,float shx,detection at){
  float nolight; 
  float lightUp;
  float smoothLight;
    nolight = lerp(1.,.1,at.day);
    nolight = lerp(nolight,.8,at.cave);
    nolight = lerp(nolight,1.5,at.night);
    nolight = lerp(nolight,.72, at.rain);
    lightUp = pow(shx,3.8);
  float3 clrTorch = float3(1.1,.22,-.8);
  result = clrTorch*nolight*lightUp;
  return result;
}

// shadow under block
float3 shadow(float3 albedo, float3 color,float shy){
  float area = smoothstep(.875, .815,shy);
  albedo = lerp(float3(1.,1.,1.),color, area);
return albedo;
}

// shadow side block
float3 shadowSide(float3 albedo, float3 shadow, float area, float a, float b){
  float limitArea = smoothstep(a, b, area);
  albedo = lerp(float3(1.,1.,1.), shadow, limitArea);
return albedo;
}

// specular
float specular(float3 wp){float3 pos; 
  pos = normalize(wp);
  float albedo = 1.-reso(length(pos.yz)*4.);
 return albedo;
}

//------water wave
float waterWave(float3 p, float tm){
      float a = noise(float2(tm*1.2+(p.x+p.z)/2., 2.5/(p.z+p.x)/2.+tm*.5));
      float b = noise(float2(tm+(p.z+p.x)/2.5, 2./(p.x+p.z)/2.5-tm*.3));
      float interp = sin(cos(p.x+p.z)+p.z+tm);
      float wave = lerp(a*.5,b*.4,-interp);
   return wave;
}

//----water normal map
float3 waterNormal(float3 p, float tm){
     float water = waterWave(p+.1,tm);
     float3 normal = float3(0.,1.,-water);
   return normal/length(normal-.5);
}

//----final water
float4 waterFinal(float4 albedo, float3 p, float3 wp, float uv, detection at){

// get water normalmap
  float getWave = dot(waterNormal(p*1.5, TIME), normalize(max(-wp, length(-wp))));

// reflect pos
float3 pos = wp; pos.z -= 8.;
float refPos = noise((-pos.xz/12.) * getWave);
refPos = lerp(refPos, saturate(refPos), max(uv, max(at.night, max(at.rain, at.cave))));

// wave color
 float4 waterc = lerp(albedo+.3, albedo+.33, at.dusk);
 waterc = lerp(waterc, albedo+.2, max(at.cave, at.night));
 waterc = lerp(waterc, lerp(albedo+.55, albedo+.22, at.night), at.rain);
 waterc = lerp(waterc, float4(1.,.6,.0,1.), mid(uv-.3));
  
// fake reflection
  float4 reflection = lerp(albedo, lerp(waterc, float4(waterc.rgb -.1, 1.), autoDistance((-wp.xz * 6.))), refPos);

// fresnel
  float fresnel = dot(normalize(-wp), float3(0., 1., 0.));
  albedo = lerp(albedo, reflection, getWave);
  if(!(FOG_CONTROL.x == 0.)){
  albedo.a = lerp(.8, albedo.a, fresnel);}
return albedo;
}

// dual fbm
float beam(float2 p){
   float2 p2 = p*.7;
   float2 basis = float2(noise( p2-time*.6), noise( p2+time*.7));
   basis = (basis-.5)*.2;
   p += basis;
return noise(mul(p,rotate(time*.185)));
}

// sunbeam
float3 sunbeam(float3 albedo, float3 wp, float3 sc, float4 fc, float sun, detection at){
   wp.yz = mul(wp.yz, float2x2(cos(time*.2), sin(time*.1), -sin(time*.1), cos(time*.2)));
   float diameter = atan(.6);
   float beamShape = abs(beam(float2(atan(wp.y/wp.z)/diameter,atan(wp.y/wp.z)/diameter)));
   float beamPos = reso(length(wp.yz)/40.);
   float beamDist = smoothstep(.1,.0, length(sc.xy)/10.);
   float3 beamClr = lerp(float3(.0,.0,.0), lerp(fc.rgb*15., float3(1.,1.,1.), at.rain),beamShape);
albedo = beamClr*beamPos*beamDist*sun*at.dusk;
return albedo;
}

// bloom sun
float3 bloomSun(float3 result, float3 hswp,float rendis,float sun,detection at){
 float bloomShape = reso(length(hswp.zy)/100.);
 float bloomPos = reso(length(hswp.x)/rendis);
 float bloom = lerp(bloomPos,0.,bloomShape);
 float3 BloomSun = lerp(float3(0.,.0,.0), float3(1.5,.8,.0),bloom);
    result = lerp(float3(0.,.0,.0),BloomSun,at.dusk*sun);
       return result;
}

// puddle
float puddle(float3 p, float3 dir1, detection at){
  float result;
  float wetPos = autoDistance(fmod(p.xz,16.)/8.-1.);
  float puddlePos = saturate(noise(p.xz/4.5));
  float  puddle = lerp(puddlePos,.0,wetPos);
  result = puddle*at.rain*(1.-at.night)*amax(-dir1.y);
return result;
}

// fake reflection on puddle
float puddleReflect(float3 p, float tm, float3 wp, detection at){float result; 
  float3 reflection = waterWave(p*3.,tm*.1)*2.;
  float3 reflectPos = reflect(normalize(-wp), reflection);
  float reflectWet = saturate(noise(reflectPos.xz*5.));
  result = reflectWet*at.rain*(1.-at.night);
return result;
}

//---- ripple
float ripple(float2 pos, float tm){
    float shape = length(pos);
    float wave = atan(pos.x+pos.y)/shape*sin((shape-.3*tm)*15.);
    float result = wave-shape;
return result;
}

//---- final ripple
float rippleFinal(float2 pos, float tm){float ripplePos;
	  ripplePos = ripple((pos*6.)-float2(1.5,1.5),tm)*amax(smoothstep(.4,.8,cos(tm*4.)));
	  ripplePos = max(ripplePos, ripple((pos*5.)-float2(3.,3.),tm*1.8)*amax(smoothstep(.2,.6, cos(tm*2.))));
	  ripplePos = max(ripplePos, ripple((pos*6.5)-float2(5.,1.),tm*1.2)*amax(smoothstep(.3,.7,cos(tm*5.))));
	  ripplePos = max(ripplePos, ripple((pos*7.5)-float2(1.,5.),tm*3.)*amax(smoothstep(.1,.2,cos(tm*3.))));
	  ripplePos = max(ripplePos, ripple((pos*4.)-float2(2.,1.),tm)*amax(smoothstep(.4,.7,cos(tm*3.7))));
	  ripplePos = max(ripplePos,ripple((pos*6.)-float2(2.,3.),tm*1.5)*amax(smoothstep(.3,.5,cos(tm*6.5))));
	  ripplePos = max(ripplePos,ripple((pos*6.5)-float2(5.5,2.5),tm*3.5)*amax(smoothstep(.1,.3,cos(tm*5.5))));
	  ripplePos = max(ripplePos,ripple((pos*5.5)-float2(2.5,4.5),tm)*amax(smoothstep(.2,.6,cos(tm*1.5))));
	  ripplePos = max(ripplePos,ripple((pos*7.)-float2(5.5,5.5),tm)*amax(smoothstep(.2,.5,cos(tm*2.5))));
return ripplePos;
}

// blink | intense torch
float3 blinkLight(float3 result,float shx,detection at){
  float randomTime = noise(ceil(TIME*6.));
  float nolight; float lightUp; float smoothLight;
     nolight = lerp(1.,.5,at.day);
     nolight = lerp(nolight,1.2,at.cave);
     nolight = lerp(nolight,1.,at.night);
     nolight = lerp(nolight,.8, at.rain);
     lightUp = pow(shx*1.15,2.7);
     smoothLight = mid(shx-.26);
float3 clrTorch = float3(1.,.48,.0);
result = lerp(clrTorch*1.2, clrTorch*.7, randomTime)*nolight*lightUp*smoothLight;
return result;
}

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{   Output Input; detection at;   //}
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD){
		discard; return;}
#endif

   float4 inColor = PSInput.color;
   float4 fc = FOG_COLOR;
   float3 AO = inColor.rgb;
   float2 fcc = FOG_CONTROL;

   float timeset = saturate((TEXTURE_1.Sample(TextureSampler1,float2(0.,1.)).r-.5)*2.);
   float lookSun = saturate((1.-fc.b)*1.5) * saturate((fc.r -.15)*1.25);
  
// timeset detection
at.day = timeset;
at.night = 1.-at.day;
at.dusk = .5-abs(.5-at.day);
at.rain = 1.-smoothstep(.3,.66,fcc.x);
at.cave = smoothstep(.5,.0, PSInput.uv1.y);
at.sunrise = at.dusk*lookSun*saturate(PSInput.wpos.x);
at.sunset = at.dusk*lookSun*saturate(-PSInput.wpos.x);

// fog detection
float morning = pow(max(min(1.-fc.b*1.2,1.), 0.),.5);
float midnight = pow(max(min(1.-fc.r*1.5,1.),.0),1.2);

   at.day *= smoothstep(.0,.7,PSInput.uv1.y);
   at.night *= smoothstep(.0,.7,PSInput.uv1.y);
   at.dusk *= smoothstep(.0,.7,PSInput.uv1.y);
   at.rain *= smoothstep(.0,.6,PSInput.uv1.y);
   at.sunset *= PSInput.uv1.y;
   at.sunrise *= PSInput.uv1.y;
   midnight *= PSInput.uv1.y;
   morning *= PSInput.uv1.y;
   lookSun *= PSInput.uv1.y;

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

if(NIGHT_VISION == OFF){
#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif
}

// detection materials
#if USE_TEXEL_AA
  Input.texSampler = texture2D_AA(TEXTURE_0,TextureSampler0,PSInput.uv0+float2(.035,.0));
  Input.texTarget = texture2D_AA(TEXTURE_0,TextureSampler0,PSInput.uv0+float2(.06245,.0));
#else
  Input.texSampler = TEXTURE_0.Sample(TextureSampler0,PSInput.uv0+float2(.035,.0));
  Input.texTarget = TEXTURE_0.Sample(TextureSampler0,PSInput.uv0+float2(.06245,.0));
#endif

//----- Change distance pos | wpos variable
// waves pos
    float3 posV = fmod(PSInput.poss,16.); 

// raysun pos
    float3 rayPos = normalize(PSInput.wpos); 
              rayPos.xz *= 2.5;

// blinn phong specular pos
    float3 blinnPos = PSInput.wpos; 
              blinnPos.zx *= 15.;
//--------------------------------------------

//------ Main Variable
// normal | direction
    float3 dir[2] = {normalize(cross(ddx(PSInput.wpos), ddy(PSInput.wpos))), 
                     normalize(cross(ddx(PSInput.poss), ddy(PSInput.poss)))};

// direction light
    float3 clrDirlight1 = lerp(float3(1.,1.,1.),lerp(float3(1.,1.,1.), lerp(float3(.4,.4,.4), float3(1.,1.,1.), mid((PSInput.uv1.x*1.5)-.5)), amax(dir[0].x)), at.day);
    float3 clrDirlight2 = lerp(float3(1.,1.,1.),lerp(float3(1.,1.,1.), lerp(float3(.4,.4,.4), float3(1.,1.,1.), mid((PSInput.uv1.x*1.5)-.5)), amax(-dir[0].x)), at.day);

// shadow under block color
  float3 clrShadows = float3(.3,.3,.4)+PSInput.uv1.x*float3(.8,.8,.8);
  clrShadows= lerp(clrShadows, float3(1.,1.,1.), at.night);
  clrShadows= lerp(clrShadows, float3(1.25,1.25,1.25), at.rain);
  clrShadows= lerp(clrShadows, float3(1.,1.,1.), at.dusk);
  clrShadows= lerp(clrShadows, float3(1.,1.,1.), autoDistance(PSInput.wpos)/RENDER_DISTANCE*2.);

// shadow side block color
  float3 clrSide = float3(.25,.25,.35)+PSInput.uv1.x*float3(.75,.75,.75);
  clrSide= lerp(clrSide, float3(1.,1.,1.), at.night);
  clrSide= lerp(clrSide, float3(1.,1.,1.), at.rain);
  clrSide= lerp(clrSide, float3(.3,.3,.3), at.dusk);

// other
    float side  = PSInput.color.g;
    float  raySun = specular(rayPos)*amax(-dir[1].y);
    float blinnPhong = specular(blinnPos)*amax(-dir[1].y);
    float getWave = waterWave(posV*1.5, TIME*.7);
    float3 Torch = light(diffuse.rgb, PSInput.uv1.x,at);
    float3 blinkTorch = blinkLight(diffuse.rgb, PSInput.uv1.x,at);
//-----------------------


#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

        // torch
        diffuse.rgb += any(bool4(isTheEnd(fc), isNether(fcc), isUnderwater(fcc,fc), BLINK_TORCH == OFF))? Torch : blinkTorch;

        // Ambient Occlusion mapping
        AO = PSInput.color.g+PSInput.color.g > PSInput.color.b + PSInput.color.r? AO : sqrt(AO);

        diffuse.rgb *= !(any(bool2(any(bool3(PSInput.color.g > PSInput.color.r, PSInput.color.g > PSInput.color.r, PSInput.color.a == 0.)), isGlow(Input))))? lerp(AO,float3(1.,1.,1.), max(1.-PSInput.uv1.y, max(float(isNether(fcc)), float(isTheEnd(fc))))) : AO;
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

if((GLOWING_ORES == ON)&&!(isNether(fcc))){
  if(isGlow(Input)){
     diffuse.rgb *= lerp(1.,lerp(5.5,7., autoTime(TIME, 1.2, 2.)), at.night);
     diffuse.rgb *= lerp(1.,lerp(12.,14., autoTime(TIME, 2.5,1.)), at.cave);
     diffuse.rgb /= lerp(1.,1.5,at.rain);
       if(any(bool2(isUnderwater(fcc,fc), isTheEnd(fc)))){
          diffuse.rgb /= 2.; }
  diffuse.rgb *= lerp(float3(1.,1.,1.), float3(.0,.0,.1), PSInput.uv1.x*1.1);}
}

if(isMetallic(Input)){
  raySun *= 0.;
  diffuse.rgb *= .8;
  diffuse.rgb += lerp(float3(.0,.0,.0),fc.rgb*float3(5.,4.,.0),blinnPhong*at.sunset);
  diffuse.rgb += lerp(float3(.0,.0,.0), fc.rgb*3., blinnPhong*at.sunrise);
  diffuse.rgb += float3(1., .4,.0)*pow(PSInput.uv1.x*1.8, 5.)*mid(PSInput.uv1.x-.7);
}

#ifndef SEASONS
if(isWater(PSInput.color)){
// water base
if(VANILLA == OFF && CLEAR_WATER == ON){
  diffuse.rgb *= float3(.0,.0,.0);
  diffuse.rgb += clrWater(diffuse.rgb,PSInput.uv1.x,at);
  //diffuse.a -= .15;
  if(!(isUnderwater(fcc,fc))){
     diffuse.a += lerp(.0,.6,at.rain);}
     diffuse.a += inColor.a*(length(PSInput.wpos)/RENDER_DISTANCE*+.7);
}

// realistic water
   if(REALISTIC_WATER == ON && VANILLA == OFF){
   diffuse = waterFinal(diffuse, PSInput.poss, PSInput.wpos, PSInput.uv1.x, at);}
   raySun *= REALISTIC_WATER == ON && VANILLA == OFF? 24.* amax(getWave) : 6.6;
}
   #endif

#if !defined (SEASONS) || !defined (ALPHA_TEST)
// underwater base
       if((isUnderwater(fcc,fc) && PSInput.color.a != 0.) &&! (isWater(PSInput.color)) && PSInput.uv1.y <.875){
    diffuse.rgb *= saturation(diffuse.rgb,1.2); // contrast
    diffuse.rgb *= float3(.8,.9,1.25);
    diffuse.rgb += float3(.2, 2.8,3.)*pow(PSInput.uv1.x*.55,2.5);

// caustic
   if(REALISTIC_WATER == ON && VANILLA == OFF){
    float3 caustic = lerp(float3(.0,.0,.0), diffuse.rgb+float3(.8,1.,1.2)*3.5, amax(getWave)) * diffuse.rgb * (1.-PSInput.uv1.x);
    diffuse.rgb += lerp(float3(.0,.0,.0), caustic, PSInput.uv1.y);}

// underwater metallic reflection
    if(isMetallic(Input)){
    diffuse.rgb += float3(1.,2.,3.)*pow(PSInput.uv1.x,2.)*mid(PSInput.uv1.x-.5);}
}
  #endif

if(SUNBEAM == ON && VANILLA == OFF){
   if(!(any(bool3(isUnderwater(fcc,fc), isTheEnd(fc), isNether(fcc))))){
   raySun *= 0.;
   diffuse.rgb += sunbeam(diffuse.rgb, PSInput.wpos, PSInput.screen, fc, lookSun, at);}
}if(!(isWater(PSInput.color)) && !(isUnderwater(fcc, fc))){

  // world assets
    _saturationSet(diffuse.rgb,at);
    _tonemapSet(diffuse.rgb,at);
    _coloringSet(diffuse.rgb,at);

}if(HIGH_SATURATION == ON){
    diffuse.rgb = saturation(diffuse.rgb,1.25);
    diffuse.rgb *= float3(1.2,1.1,1.);
    if(any(bool2(PSInput.color.g > PSInput.color.r, PSInput.color.b < PSInput.color.g))){
    diffuse.rgb *= float3(.9,.9,.9);}
}if(PSInput.color.a == 0.){
     diffuse.rgb *= float3(1.3,1.45,1.3);
     if(SHADOW_SIDE_BLOCK == ON && VANILLA == OFF){
     side = PSInput.color.g*1.2;
     clrSide *= .0; 
   }
}

if(DIRLIGHT == ON && VANILLA == OFF){
if(!(PSInput.color.a == 0.)){
  //light dir at day
     if(at.rain <.8 && !isUnderwater(fcc,fc)){
     if(!(all(bool2(PSInput.color.g > PSInput.color.r, PSInput.color.a>0.)))){
     diffuse.rgb *= clrDirlight2;
     diffuse.rgb *= lerp(lerp(float3(1.,1.,1.),float3(1.4,1.4,1.4), amax(dir[0].z)),float3(1.,1.,1.), max(autoDistance(PSInput.wpos/RENDER_DISTANCE), max(at.cave, at.night)));}
     diffuse.rgb *= clrDirlight1;
     diffuse.rgb *= lerp(lerp(float3(1.,1.,1.),float3(1.4,1.4,1.4), amax(-dir[0].z)),float3(1.,1.,1.), max(autoDistance(PSInput.wpos/RENDER_DISTANCE), max(at.cave, at.night)));}

  //light dir at dusk
     if(at.rain <.2 && !isUnderwater(fcc,fc)){
     if(!(all(bool2(PSInput.color.g > PSInput.color.r, PSInput.color.a>0.)))){
     diffuse.rgb *= lerp(lerp(float3(1.,1.,1.), lerp(float3(1.,1.,1.), lerp(float3(2.,1.7,.5), float3(1.,1.,1.), PSInput.uv1.x), amax(dir[0].z)), at.dusk), float3(1.,1.,1.), autoDistance(PSInput.wpos/RENDER_DISTANCE*2.));}
     diffuse.rgb *= lerp(lerp(float3(1.,1.,1.), lerp(float3(1.,1.,1.), lerp(float3(2.,1.7,.5), float3(1.,1.,1.), PSInput.uv1.x), amax(-dir[0].z)), at.dusk), float3(1.,1.,1.), autoDistance(PSInput.wpos/RENDER_DISTANCE*2.));}
  }
}

if(SHADOW_UNDER_BLOCK == ON && VANILLA == OFF){
   if(!(isUnderwater(fcc,fc))){
      diffuse.rgb *= shadow(diffuse.rgb, clrShadows, PSInput.uv1.y);}
}

#ifndef ALPHA_TEST
if(SHADOW_SIDE_BLOCK == ON && VANILLA == OFF){
  if(!isWater(PSInput.color)){
    diffuse.rgb *= lerp(lerp(float3(1.,1.,1.), shadowSide(diffuse.rgb, clrSide, side, .6, .5), abs(dir[0].z)), float3(1.,1.,1.), smoothstep(.875, .815, PSInput.uv1.y));}
}
  #endif

//----- Coloring Other Dimension
// nether color
   if(isNether(fcc)){
   diffuse.rgb *= float3(1.2,1.2,1.2);
   if(GLOWING_ORES == ON){
       if(isGlow(Input)){
       diffuse.rgb *= 3.*(1.-PSInput.uv1.x);
        } //>1
       } //>2
      } //>3

// base | torch color end
  if(isTheEnd(fc)){
     float dist = amax(PSInput.uv1.x-.3);
     float3 TorchEnd = float3(.05,.0,1.8)*pow(PSInput.uv1.x*.8,2.)*dist;
     diffuse.rgb += TorchEnd;

// the end color
   if(GALAXY == OFF && VANILLA == OFF){
      diffuse.rgb *= float3(1.,.9,1.1);
      float fogPos = clamp(length(PSInput.wpos)/RENDER_DISTANCE*3.,.8,1.6);
      float3 fogEnd = lerp(float3(0.,.0,0.), float3(.15,.115,.24),fogPos);
      diffuse.rgb += fogEnd;

              }else{

      float fogPos2 = saturate(length(PSInput.wpos)/RENDER_DISTANCE);
      float3 fogEnd2 = lerp(float3(0.,0.,0.), float3(.05,.0,.2),fogPos2);
      diffuse.rgb += fogEnd2;
      diffuse.rgb *= float3(.7,.7,1.5);
     }
  }
//--------------------------------

if(WET_EFFECT == ON && VANILLA == OFF){
// wet distance
    float rendering = autoPos(PSInput.wpos.xz/25.);

// puddle pos
    float puddles = puddle(PSInput.poss*1.015, dir[0],at);

//---- wet effect
    // >1 wet blocks
    float4 wet = TEXTURE_0.Sample(TextureSampler0,diffuse.xz);

    //>2 puddle outside
    float3 puddle1 = lerp(float3(.0,.0,.0), float3(2.5,2.5,2.5), lerp(puddle(PSInput.poss,dir[0],at),0., rendering));

    //>3 puddle inside
    float3 puddle2 = lerp(float3(0.,0.,0.), lerp(float3(1.24,1.29,1.3)*3., float3(2.,2.,2.)*4., pow(PSInput.uv1.x,5.)), lerp(puddles, 0., rendering));

    //>4 fake reflection on puddle
    float3 reflection = lerp(diffuse.rgb, lerp(diffuse.rgb, lerp(lerp(diffuse.rgb, puddleReflect(PSInput.poss, TIME*.35, PSInput.wpos*.5, at)*float3(9.,9.,9.), autoPos(PSInput.wpos.xz/5.)),diffuse.rgb, rendering), puddles), amax(-dir[1].y));

   //>5 rain splash
     float splash = lerp(0., lerp(reso(rippleFinal(frac(PSInput.poss.zx*.6), TIME))*amax(-dir[0].y)*at.rain*PSInput.uv1.y, 0., rendering), puddles);
//--------------------------------------

  if(!(any(bool2(isUnderwater(fcc,fc),isWater(PSInput.color))))){
     diffuse.rgb = reflection;
     diffuse += wet*.15*at.rain*(1.-at.night);
     diffuse.rgb -= puddle1;
     diffuse.rgb += puddle2;
     diffuse.rgb += splash;
  }
}

if(RAY_SUN == ON && VANILLA == OFF){
   if(at.rain <.2){
   diffuse.rgb += lerp(float3(.0,.0,.0), fc.rgb*float3(1.,.9,.0), raySun*lookSun*at.dusk);}
}

//------ Main fog 
#ifdef LOW_PRECISION
// atmosphere
    if(ATMOSPHERE == ON){
    float Atmosphere = length(-PSInput.wpos.xz*1.5) / FAR_CHUNKS_DISTANCE*.2;
    Atmosphere = clamp(Atmosphere, .15, 1.)*(1.-at.night)*(1.-at.cave);
    diffuse.rgb = lerp(diffuse.rgb,fc.rgb*float3(1.,1.1,1.2),Atmosphere);

// fog rain
    float foggyRain = saturate(length(-PSInput.wpos.xyz*3.)/RENDER_DISTANCE*1.5);
    float3 foggyRainON = lerp(float3(.0,.0,.0), lerp(float3(.55,.55,.55), float3(.05,.05,.05), midnight),foggyRain);
    diffuse.rgb += lerp(float3(.0,.0,.0),foggyRainON,at.rain);

// fog dusk
    float foggyDusk = saturate(length(PSInput.wpos) / RENDER_DISTANCE *3.)*(1.-at.night)*(1.-at.cave);
    float3 foggyDuskON = lerp(diffuse.rgb,fc.rgb*float3(1.5,1.5,1.5),foggyDusk);
    diffuse.rgb = lerp(diffuse.rgb, foggyDuskON,morning);

// fog night
    float foggyNight = saturate(length(PSInput.wpos) / RENDER_DISTANCE*3.);
    float3 foggyNightON = lerp(float3(0.,0.,0.),float3(.1,.1,.1),foggyNight);
    diffuse.rgb += lerp(float3(.0,.0,.0), foggyNightON,midnight);}

// ground fog
    if(GROUND_FOG == ON){
    if(at.rain <.2){
    float fogDusk = saturate(length(PSInput.wpos)/FAR_CHUNKS_DISTANCE);
    fogDusk = reso(fogDusk*-(PSInput.wpos.y-12.)/15.);
    float3 fogDuskON = lerp(diffuse.rgb, fc.rgb*1.8,fogDusk);
    diffuse.rgb = lerp(diffuse.rgb,fogDuskON,at.dusk);}}
    #endif
//------------------

if(BLOOM_SUN == ON && GROUND_FOG == OFF){
  diffuse.rgb += bloomSun(diffuse.rgb, PSInput.wpos,RENDER_DISTANCE, lookSun,at);
}

#ifdef FOG
if(at.night <.2){
  diffuse.rgb += lerp(float3(.0,.0,.0), float3(.1,.1,.1),PSInput.fogColor.a);}
  if(isNether(fcc) || fcc.x == 0.){
    diffuse.rgb = lerp(diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a);}
  #endif

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}