#include "ShaderConstants.fxh"
#include "lib/option.fxh"

struct VS_Input {
	float3 position : POSITION;
	float4 color : COLOR;
	float2 uv0 : TEXCOORD_0;
	float2 uv1 : TEXCOORD_1;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
};

struct PS_Input {
	float4 position : SV_Position;
        float3 poss : POSS;
        float3 wpos : WPOS;
        float3 screen : SCREEN;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};


static const float rA = 1.0;
static const float rB = 1.0;
static const float3 UNIT_Y = float3(0, 1, 0);
static const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange


ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput)
{
#ifndef BYPASS_PIXEL_SHADER
	PSInput.uv0 = VSInput.uv0;
	PSInput.uv1 = VSInput.uv1;
	PSInput.color = VSInput.color;
#endif

#ifdef AS_ENTITY_RENDERER
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
		PSInput.position = mul(WORLDVIEWPROJ_STEREO[i], float4(VSInput.position, 1));
	#else
		PSInput.position = mul(WORLDVIEWPROJ, float4(VSInput.position, 1));
	#endif
		float3 worldPos = PSInput.position;
#else
		float3 worldPos = (VSInput.position.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
	
		// Transform to view space before projection instead of all at once to avoid floating point errors
		// Not required for entities because they are already offset by camera translation before rendering
		// World position here is calculated above and can get huge
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
	
		PSInput.position = mul(WORLDVIEW_STEREO[i], float4(worldPos, 1 ));
		PSInput.position = mul(PROJ_STEREO[i], PSInput.position);
	
	#else
		PSInput.position = mul(WORLDVIEW, float4( worldPos, 1 ));
		PSInput.position = mul(PROJ, PSInput.position);
	#endif

               PSInput.wpos = worldPos.xyz;
               PSInput.poss = VSInput.position.xyz;
               PSInput.screen.xy = PSInput.position.xy / (PSInput.position.z + 1.0);

#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
		PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
		PSInput.renTarget_id = VSInput.instanceID;
#endif
///// find distance from the camera

//----- Variables
   float3 p = fmod(PSInput.poss,16.); // no chunk
   float rain = 1.-smoothstep(.3,.66,FOG_CONTROL.x);
   float water_wave = sin(TIME*5.+p.z+p.y+p.x+cos(TIME*3.+p.x+p.y+p.z));
   float grass_wave = sin(TIME*2.5+ p.x + p.y +p.z+cos(TIME+ p.x + p.z));
   float wave_rain = sin(TIME*5. + p.y + p.z +p.x + cos(TIME*4.+p.x+p.z+p.y))*rain;
   float underwater_wave = sin(TIME*2.+p.x+p.z+p.y+p.z);
//--------------

if(LEAVES_WAVING == ON && VANILLA == OFF){
#ifdef ALPHA_TEST
// waving grass
   if(PSInput.color.g>PSInput.color.r&&PSInput.color.a>0. || PSInput.color.r > PSInput.color.g  && PSInput.color.g > PSInput.color.b && PSInput.color.a != 0.){
   PSInput.position.x += grass_wave*.05;
   PSInput.position.x += wave_rain*.03;}

// waving leaves
    if(PSInput.color.a == 0.){
    PSInput.position.x += grass_wave*.04;
    PSInput.position.x += wave_rain*.06;}
   #endif
}

#if !defined SEASONS &&! defined ALPHA_TEST
// water waving
  if(WATER_WAVING == ON && VANILLA == OFF){
  if(isWater(PSInput.color)){
  PSInput.position.y += water_wave*.08;}
}

// under water waving
  if(UNDER_WATER_WAVING == ON && VANILLA == OFF){
  if(isUnderwater(FOG_CONTROL, FOG_COLOR)){
  PSInput.position.xy += underwater_wave*.045;}
}
  #endif


#if defined(FOG) || defined(BLEND)
	#ifdef FANCY
		float3 relPos = -worldPos;
		float cameraDepth = length(relPos);
	#else
		float cameraDepth = PSInput.position.z;
	#endif
#endif

	///// apply fog

#ifdef FOG
	float len = cameraDepth / RENDER_DISTANCE;
#ifdef ALLOW_FADE
	len += RENDER_CHUNK_FOG_ALPHA.r;
#endif

	PSInput.fogColor.rgb = FOG_COLOR.rgb;
	PSInput.fogColor.a = clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);

#endif

///// blended layer (mostly water) magic
#ifdef BLEND
	//Mega hack: only things that become opaque are allowed to have vertex-driven transparency in the Blended layer...
	//to fix this we'd need to find more space for a flag in the vertex format. color.a is the only unused part
	bool shouldBecomeOpaqueInTheDistance = VSInput.color.a < 0.95;
	if(shouldBecomeOpaqueInTheDistance) {
		#ifdef FANCY  /////enhance water
			float cameraDist = cameraDepth / FAR_CHUNKS_DISTANCE;
		#else
			float3 relPos = -worldPos.xyz;
			float camDist = length(relPos);
			float cameraDist = camDist / FAR_CHUNKS_DISTANCE;
		#endif //FANCY
		
		float alphaFadeOut = clamp(cameraDist, 0.0, 1.0);
		PSInput.color.a = lerp(VSInput.color.a, 1.0, alphaFadeOut);
	}
#endif

}
